@@ -1,14 +1,12 @@
 #ifndef __DMENU_DIALOG_H__
 #define __DMENU_DIALOG_H__
 
-
 /**
- * @param input Pointer to the user-input string.
- *
  * dmenu dialog.
  *
  * @returns TRUE if script was successful.
  */
-int dmenu_switcher_dialog ( char **input );
+int dmenu_switcher_dialog ( void );
 
+void print_dmenu_options ( void );
 #endif
