@@ -1,5 +1,5 @@
-#ifndef __HELPER_H__
-#define __HELPER_H__
+#ifndef ROFI_HELPER_H
+#define ROFI_HELPER_H
 #include "rofi.h"
 /**
  * @param string The input string.
@@ -16,7 +16,7 @@ int helper_parse_setup ( char * string, char ***output, int *length, ... );
 /**
  * Implementation of fgets with custom separator.
  */
-char* fgets_s ( char* s, int n, FILE *iop, char sep );
+char* fgets_s ( char* s, unsigned int n, FILE *iop, char sep );
 
 /**
  * @param token The string for which we want a collation key.
@@ -71,7 +71,6 @@ int find_arg_uint ( const char * const key, unsigned int *val );
  */
 int find_arg_int ( const char * const key, int *val );
 
-
 /**
  * @param key The key to search for
  * @param val Pointer to the string to set to the key value (if found)
@@ -81,7 +80,6 @@ int find_arg_int ( const char * const key, int *val );
  * @returns TRUE if key was found and val was set.
  */
 int find_arg_str ( const char * const key, char** val );
-int find_arg_str_alloc ( const char * const key, char** val );
 
 /**
  * @param key The key to search for
@@ -104,7 +102,7 @@ int find_arg ( const char * const key );
  *
  * @returns 1 when matches, 0 otherwise
  */
-int token_match ( char **tokens, const char *input, int case_sensitive,
+int token_match ( char **tokens, const char *input, int not_ascii, int case_sensitive,
                   __attribute__( ( unused ) ) unsigned int index,
                   __attribute__( ( unused ) ) Switcher * data );
 
@@ -121,9 +119,14 @@ int execute_generator ( const char * cmd ) __attribute__( ( nonnull ) );
 /**
  * @param pidfile The pidfile to create.
  *
- * Calls exit (1) when failed.
+ * returns file descriptor (or -1 when failed)
  */
-void create_pid_file ( const char *pidfile );
+int create_pid_file ( const char *pidfile );
+
+/**
+ * Remove pid file
+ */
+void remove_pid_file ( int fd );
 
 /**
  * Do some input validation, especially the first few could break things.
@@ -131,7 +134,7 @@ void create_pid_file ( const char *pidfile );
  *
  * This functions exits the program with 1 when it finds an invalid configuration.
  */
-void config_sanity_check ( void );
+void config_sanity_check ( Display *display );
 
 /**
  * @param arg string to parse.
@@ -149,4 +152,11 @@ char helper_parse_char ( const char *arg );
  * Set the application arguments.
  */
 void cmd_set_arguments ( int argc, char **argv );
-#endif // __HELPER_H__
+
+/**
+ * @param str a UTF8 string
+ * @return 1 if the string contains any non-ascii codepoints
+ */
+int is_not_ascii ( const char *str );
+
+#endif // ROFI_HELPER_H
