@@ -1,5 +1,5 @@
-#ifndef __KEYB_H__
-#define __KEYB_H__
+#ifndef ROFI_KEYB_H
+#define ROFI_KEYB_H
 
 typedef enum _KeyBindingAction
 {
@@ -23,15 +23,6 @@ typedef enum _KeyBindingAction
     MODE_PREVIOUS,
     TOGGLE_CASE_SENSITIVITY,
     DELETE_ENTRY,
-    CUSTOM_1,
-    CUSTOM_2,
-    CUSTOM_3,
-    CUSTOM_4,
-    CUSTOM_5,
-    CUSTOM_6,
-    CUSTOM_7,
-    CUSTOM_8,
-    CUSTOM_9,
     ROW_LEFT,
     ROW_RIGHT,
     ROW_UP,
@@ -43,10 +34,28 @@ typedef enum _KeyBindingAction
     ROW_LAST,
     ROW_SELECT,
     CANCEL,
+    CUSTOM_1,
+    CUSTOM_2,
+    CUSTOM_3,
+    CUSTOM_4,
+    CUSTOM_5,
+    CUSTOM_6,
+    CUSTOM_7,
+    CUSTOM_8,
+    CUSTOM_9,
+    CUSTOM_10,
+    CUSTOM_11,
+    CUSTOM_12,
+    CUSTOM_13,
+    CUSTOM_14,
+    CUSTOM_15,
+    CUSTOM_16,
+    CUSTOM_17,
+    CUSTOM_18,
+    CUSTOM_19,
     NUM_ABE
 } KeyBindingAction;
 
-
 /**
  * Parse the keybindings.
  * This should be called after the setting system is initialized.
@@ -69,4 +78,4 @@ void cleanup_abe ( void );
  * @returns TRUE if key combo matches, FALSE otherwise.
  */
 int abe_test_action ( KeyBindingAction action, unsigned int mask, KeySym key );
-#endif // __KEYB_H__
+#endif // ROFI_KEYB_H
