@@ -1,5 +1,5 @@
-#ifndef __XRMOPTIONS_H__
-#define __XRMOPTIONS_H__
+#ifndef ROFI_XRMOPTIONS_H
+#define ROFI_XRMOPTIONS_H
 
 // Big thanks to Sean Pringle for this code.
 // This maps xresource options to config structure.
@@ -20,7 +20,6 @@ typedef enum
  */
 void config_parse_xresource_options ( Display *display );
 
-
 /**
  * Parse commandline options.
  */
@@ -51,5 +50,6 @@ void xresource_dump ( void );
 /**
  * Add option to the dynamic option parser.
  */
-void config_parser_add_option ( XrmOptionType type, const char *key, void **value );
+void config_parser_add_option ( XrmOptionType type, const char *key, void **value, const char *comment );
+void print_options ( void );
 #endif
