@@ -45,8 +45,9 @@
 
 #define RUN_CACHE_FILE    "rofi-2.runcache"
 
-static inline void execsh ( const char *cmd, int run_in_term )
+static inline int execsh ( const char *cmd, int run_in_term )
 {
+    int  retv   = TRUE;
     char **args = NULL;
     int  argc   = 0;
     if ( run_in_term ) {
@@ -56,20 +57,19 @@ static inline void execsh ( const char *cmd, int run_in_term )
         helper_parse_setup ( config.run_command, &args, &argc, "{cmd}", cmd, NULL );
     }
     GError *error = NULL;
-    g_spawn_async ( NULL, args, NULL,
-                    G_SPAWN_SEARCH_PATH,
-                    NULL, NULL, NULL, &error );
+    g_spawn_async ( NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error );
     if ( error != NULL ) {
-        char *msg = g_strdup_printf ( "Failed to execute: '%s'\nError: '%s'", cmd,
-                                      error->message );
+        char *msg = g_strdup_printf ( "Failed to execute: '%s'\nError: '%s'", cmd, error->message );
         error_dialog ( msg, FALSE  );
         g_free ( msg );
         // print error.
         g_error_free ( error );
+        retv = FALSE;
     }
 
     // Free the args list.
     g_strfreev ( args );
+    return retv;
 }
 
 // execute sub-process
@@ -79,18 +79,17 @@ static void exec_cmd ( const char *cmd, int run_in_term )
         return;
     }
 
+    if (  execsh ( cmd, run_in_term ) ) {
+        /**
+         * This happens in non-critical time (After launching app)
+         * It is allowed to be a bit slower.
+         */
+        char *path = g_strdup_printf ( "%s/%s", cache_dir, RUN_CACHE_FILE );
 
-    execsh ( cmd, run_in_term );
+        history_set ( path, cmd );
 
-    /**
-     * This happens in non-critical time (After launching app)
-     * It is allowed to be a bit slower.
-     */
-    char *path = g_strdup_printf ( "%s/%s", cache_dir, RUN_CACHE_FILE );
-
-    history_set ( path, cmd );
-
-    g_free ( path );
+        g_free ( path );
+    }
 }
 // execute sub-process
 static void delete_entry ( const char *cmd )
@@ -155,10 +154,12 @@ static char ** get_apps_external ( char **retv, unsigned int *length, unsigned i
                 retv              = g_realloc ( retv, ( ( *length ) + 2 ) * sizeof ( char* ) );
                 retv[( *length )] = g_strdup ( buffer );
 
-
                 ( *length )++;
             }
-            fclose ( inp );
+            if ( fclose ( inp ) != 0 ) {
+                fprintf ( stderr, "Failed to close stdout off executor script: '%s'\n",
+                          strerror ( errno ) );
+            }
         }
     }
     retv[( *length ) ] = NULL;
@@ -178,28 +179,22 @@ static char ** get_apps ( unsigned int *length )
         return NULL;
     }
 
-
     path = g_strdup_printf ( "%s/%s", cache_dir, RUN_CACHE_FILE );
     retv = history_get_list ( path, length );
     g_free ( path );
     // Keep track of how many where loaded as favorite.
     num_favorites = ( *length );
 
-
     path = g_strdup ( getenv ( "PATH" ) );
 
-    for ( const char *dirname = strtok ( path, ":" );
-          dirname != NULL;
-          dirname = strtok ( NULL, ":" ) ) {
+    for ( const char *dirname = strtok ( path, ":" ); dirname != NULL; dirname = strtok ( NULL, ":" ) ) {
         DIR *dir = opendir ( dirname );
 
         if ( dir != NULL ) {
             struct dirent *dent;
 
             while ( ( dent = readdir ( dir ) ) != NULL ) {
-                if ( dent->d_type != DT_REG &&
-                     dent->d_type != DT_LNK &&
-                     dent->d_type != DT_UNKNOWN ) {
+                if ( dent->d_type != DT_REG && dent->d_type != DT_LNK && dent->d_type != DT_UNKNOWN ) {
                     continue;
                 }
                 // Skip dot files.
@@ -254,9 +249,10 @@ static char ** get_apps ( unsigned int *length )
         }
     }
 
-
     if ( ( *length ) > num_favorites ) {
-        g_qsort_with_data ( &retv[num_favorites], ( *length ) - num_favorites, sizeof ( char* ), sort_func, NULL );
+        g_qsort_with_data ( &retv[num_favorites], ( *length ) - num_favorites, sizeof ( char* ),
+                            sort_func,
+                            NULL );
     }
     // Reduce array length;
     ( *length ) -= removed;
@@ -271,7 +267,6 @@ typedef struct _RunModePrivateData
     unsigned int cmd_list_length;
 } RunModePrivateData;
 
-
 static void run_mode_init ( Switcher *sw )
 {
     if ( sw->private_data == NULL ) {
@@ -287,11 +282,14 @@ static char ** run_mode_get_data ( unsigned int *length, Switcher *sw )
         rmpd->cmd_list_length = 0;
         rmpd->cmd_list        = get_apps ( &( rmpd->cmd_list_length ) );
     }
-    *length = rmpd->cmd_list_length;
+    if ( length != NULL ) {
+        *length = rmpd->cmd_list_length;
+    }
     return rmpd->cmd_list;
 }
 
-static SwitcherMode run_mode_result ( int mretv, char **input, unsigned int selected_line, Switcher *sw )
+static SwitcherMode run_mode_result ( int mretv, char **input, unsigned int selected_line,
+                                      Switcher *sw )
 {
     RunModePrivateData *rmpd = (RunModePrivateData *) sw->private_data;
     SwitcherMode       retv  = MODE_EXIT;
@@ -325,7 +323,6 @@ static SwitcherMode run_mode_result ( int mretv, char **input, unsigned int sele
     return retv;
 }
 
-
 static void run_mode_destroy ( Switcher *sw )
 {
     RunModePrivateData *rmpd = (RunModePrivateData *) sw->private_data;
@@ -338,14 +335,12 @@ static void run_mode_destroy ( Switcher *sw )
 
 static const char *mgrv ( unsigned int selected_line, void *sw, G_GNUC_UNUSED int *state )
 {
-    RunModePrivateData *rmpd = ( (Switcher *) sw )->private_data;
-    return rmpd->cmd_list[selected_line];
+    return run_mode_get_data ( NULL, sw )[selected_line];
 }
 
 Switcher run_mode =
 {
     .name         = "run",
-    .tb           = NULL,
     .keycfg       = NULL,
     .keystr       = NULL,
     .modmask      = AnyModifier,
