@@ -25,7 +25,6 @@
  *
  */
 
-
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,14 +33,11 @@
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
+#include <errno.h>
 #include "rofi.h"
 #include "dialogs/script.h"
 #include "helper.h"
 
-
-
-
-
 static char **get_script_output ( const char *command, unsigned int *length )
 {
     char **retv = NULL;
@@ -64,7 +60,10 @@ static char **get_script_output ( const char *command, unsigned int *length )
 
                 ( *length )++;
             }
-            fclose ( inp );
+            if ( fclose ( inp ) != 0 ) {
+                fprintf ( stderr, "Failed to close stdout off executor script: '%s'\n",
+                          strerror ( errno ) );
+            }
         }
     }
     return retv;
@@ -98,7 +97,6 @@ typedef struct _ScriptModePrivateData
     unsigned int cmd_list_length;
 } ScriptModePrivateData;
 
-
 static void script_mode_init ( Switcher *sw )
 {
     if ( sw->private_data == NULL ) {
@@ -117,7 +115,8 @@ static char ** script_mode_get_data ( unsigned int *length, Switcher *sw )
     return rmpd->cmd_list;
 }
 
-static SwitcherMode script_mode_result ( int mretv, char **input, unsigned int selected_line, Switcher *sw )
+static SwitcherMode script_mode_result ( int mretv, char **input, unsigned int selected_line,
+                                         Switcher *sw )
 {
     ScriptModePrivateData *rmpd      = (ScriptModePrivateData *) sw->private_data;
     SwitcherMode          retv       = MODE_EXIT;
@@ -140,7 +139,6 @@ static SwitcherMode script_mode_result ( int mretv, char **input, unsigned int s
         new_list = execute_executor ( sw, *input, &new_length );
     }
 
-
     // If a new list was generated, use that an loop around.
     if ( new_list != NULL ) {
         g_strfreev ( rmpd->cmd_list );
@@ -198,8 +196,7 @@ Switcher *script_switcher_parse_setup ( const char *str )
 
         return sw;
     }
-    fprintf ( stderr, "The script command '%s' has %u options, but needs 2: <name>:<script>.\n",
-              str, index );
+    fprintf ( stderr, "The script command '%s' has %u options, but needs 2: <name>:<script>.\n", str, index );
     script_switcher_free ( sw );
     return NULL;
 }
