@@ -25,6 +25,9 @@
  *
  */
 #include <config.h>
+
+#ifdef WINDOW_MODE
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -41,14 +44,14 @@
 #include "i3-support.h"
 #include "dialogs/window.h"
 
+#define WINLIST             32
 
-#define WINLIST        32
-
-#define CLIENTTITLE    100
-#define CLIENTCLASS    50
-#define CLIENTNAME     50
-#define CLIENTSTATE    10
-#define CLIENTROLE     50
+#define CLIENTTITLE         100
+#define CLIENTCLASS         50
+#define CLIENTNAME          50
+#define CLIENTSTATE         10
+#define CLIENTWINDOWTYPE    10
+#define CLIENTROLE          50
 
 // a manageable window
 typedef struct
@@ -61,6 +64,8 @@ typedef struct
     char              role[CLIENTROLE];
     int               states;
     Atom              state[CLIENTSTATE];
+    int               window_types;
+    Atom              window_type[CLIENTWINDOWTYPE];
     workarea          monitor;
     int               active;
     int               demands;
@@ -168,8 +173,12 @@ static int winlist_find ( winlist *l, Window w )
  */
 static void x11_cache_create ( void )
 {
-    cache_client = winlist_new ();
-    cache_xattr  = winlist_new ();
+    if ( cache_client == NULL ) {
+        cache_client = winlist_new ();
+    }
+    if ( cache_xattr == NULL ) {
+        cache_xattr = winlist_new ();
+    }
 }
 
 /**
@@ -178,7 +187,9 @@ static void x11_cache_create ( void )
 static void x11_cache_free ( void )
 {
     winlist_free ( cache_xattr );
+    cache_xattr = NULL;
     winlist_free ( cache_client );
+    cache_client = NULL;
 }
 
 /**
@@ -219,6 +230,16 @@ static int client_has_state ( client *c, Atom state )
 
     return 0;
 }
+static int client_has_window_type ( client *c, Atom type )
+{
+    for ( int i = 0; i < c->window_types; i++ ) {
+        if ( c->window_type[i] == type ) {
+            return 1;
+        }
+    }
+
+    return 0;
+}
 
 static client* window_client ( Display *display, Window win )
 {
@@ -248,6 +269,7 @@ static client* window_client ( Display *display, Window win )
 
     c->states = window_get_atom_prop ( display, win, netatoms[_NET_WM_STATE], c->state, CLIENTSTATE );
 
+    c->window_types = window_get_atom_prop ( display, win, netatoms[_NET_WM_WINDOW_TYPE], c->window_type, CLIENTWINDOWTYPE );
     char *name;
 
     if ( ( name = window_get_text_prop ( display, c->window, netatoms[_NET_WM_NAME] ) ) && name ) {
@@ -300,6 +322,7 @@ typedef struct _SwitcherModePrivateData
 } SwitcherModePrivateData;
 
 static int window_match ( char **tokens, __attribute__( ( unused ) ) const char *input,
+                          __attribute__( ( unused ) ) int not_ascii,
                           int case_sensitive, unsigned int index, Switcher *sw )
 {
     SwitcherModePrivateData *rmpd = (SwitcherModePrivateData *) sw->private_data;
@@ -316,19 +339,19 @@ static int window_match ( char **tokens, __attribute__( ( unused ) ) const char
             // e.g. when searching 'title element' and 'class element'
             char *ftokens[2] = { tokens[j], NULL };
             if ( !test && c->title[0] != '\0' ) {
-                test = token_match ( ftokens, c->title, case_sensitive, 0, NULL );
+                test = token_match ( ftokens, c->title, is_not_ascii ( c->title ), case_sensitive, 0, NULL );
             }
 
             if ( !test && c->class[0] != '\0' ) {
-                test = token_match ( ftokens, c->class, case_sensitive, 0, NULL );
+                test = token_match ( ftokens, c->class, is_not_ascii ( c->class ), case_sensitive, 0, NULL );
             }
 
             if ( !test && c->role[0] != '\0' ) {
-                test = token_match ( ftokens, c->role, case_sensitive, 0, NULL );
+                test = token_match ( ftokens, c->role, is_not_ascii ( c->role ), case_sensitive, 0, NULL );
             }
 
             if ( !test && c->name[0] != '\0' ) {
-                test = token_match ( ftokens, c->name, case_sensitive, 0, NULL );
+                test = token_match ( ftokens, c->name, is_not_ascii ( c->name ), case_sensitive, 0, NULL );
             }
 
             if ( test == 0 ) {
@@ -348,7 +371,7 @@ static void window_mode_init ( Switcher *sw )
     }
 }
 
-static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
+static char ** _window_mode_get_data ( unsigned int *length, Switcher *sw, unsigned int cd )
 {
     SwitcherModePrivateData *pd = (SwitcherModePrivateData *) sw->private_data;
     if ( !pd->init ) {
@@ -367,14 +390,18 @@ static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
         pd->config_i3_mode = i3_support_initialize ( display );
 
         // Get the active window so we can highlight this.
-        if ( !( window_get_prop ( display, root, netatoms[_NET_ACTIVE_WINDOW], &type,
-                                  &count, &curr_win_id, sizeof ( Window ) )
+        if ( !( window_get_prop ( display, root, netatoms[_NET_ACTIVE_WINDOW], &type, &count, &curr_win_id, sizeof ( Window ) )
                 && type == XA_WINDOW && count > 0 ) ) {
             curr_win_id = 0;
         }
 
-        if ( window_get_prop ( display, root, netatoms[_NET_CLIENT_LIST_STACKING],
-                               &type, &nwins, wins, 100 * sizeof ( Window ) )
+        // Get the current desktop.
+        unsigned long current_desktop = 0;
+        if ( !window_get_cardinal_prop ( display, root, netatoms[_NET_CURRENT_DESKTOP], &current_desktop, 1 ) ) {
+            current_desktop = 0;
+        }
+
+        if ( window_get_prop ( display, root, netatoms[_NET_CLIENT_LIST_STACKING], &type, &nwins, wins, 100 * sizeof ( Window ) )
              && type == XA_WINDOW ) {
             char          pattern[50];
             int           i;
@@ -384,22 +411,18 @@ static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
             // if we happen to have a window destroyed while we're working...
             pd->ids = winlist_new ();
 
-
-
             // calc widths of fields
             for ( i = nwins - 1; i > -1; i-- ) {
                 client *c;
 
                 if ( ( c = window_client ( display, wins[i] ) )
                      && !c->xattr.override_redirect
+                     && !client_has_window_type ( c, netatoms[_NET_WM_WINDOW_TYPE_DOCK] )
+                     && !client_has_window_type ( c, netatoms[_NET_WM_WINDOW_TYPE_DESKTOP] )
                      && !client_has_state ( c, netatoms[_NET_WM_STATE_SKIP_PAGER] )
                      && !client_has_state ( c, netatoms[_NET_WM_STATE_SKIP_TASKBAR] ) ) {
                     classfield = MAX ( classfield, strlen ( c->class ) );
 
-                    // In i3 mode, skip the i3bar completely.
-                    if ( pd->config_i3_mode && strstr ( c->class, "i3bar" ) != NULL ) {
-                        continue;
-                    }
                     if ( client_has_state ( c, netatoms[_NET_WM_STATE_DEMANDS_ATTENTION] ) ) {
                         c->demands = TRUE;
                     }
@@ -422,7 +445,8 @@ static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
                 sprintf ( pattern, "%%-%ds   %%s", MAX ( 5, classfield ) );
             }
             else{
-                sprintf ( pattern, "%%-%ds  %%-%ds   %%s", desktops < 10 ? 1 : 2, MAX ( 5, classfield ) );
+                sprintf ( pattern, "%%-%ds  %%-%ds   %%s", desktops < 10 ? 1 : 2,
+                          MAX ( 5, classfield ) );
             }
             pd->cmd_list = g_malloc0_n ( ( pd->ids->len + 1 ), sizeof ( char* ) );
 
@@ -438,14 +462,18 @@ static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
                     desktop[0] = 0;
                     char          *line = g_malloc ( strlen ( c->title ) + strlen ( c->class ) + classfield + 50 );
                     if ( !pd->config_i3_mode ) {
-                        // find client's desktop. This is zero-based, so we adjust by since most
-                        // normal people don't think like this :-)
+                        // find client's desktop.
                         if ( !window_get_cardinal_prop ( display, c->window, netatoms[_NET_WM_DESKTOP], &wmdesktop, 1 ) ) {
+                            // Assume the client is on all desktops.
                             wmdesktop = 0xFFFFFFFF;
                         }
+                        else if ( cd && wmdesktop != current_desktop ) {
+                            g_free ( line );
+                            continue;
+                        }
 
                         if ( wmdesktop < 0xFFFFFFFF ) {
-                            sprintf ( desktop, "%d", (int) wmdesktop + 1 );
+                            sprintf ( desktop, "%d", (int) wmdesktop );
                         }
 
                         sprintf ( line, pattern, desktop, c->class, c->title );
@@ -463,7 +491,17 @@ static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
     *length = pd->cmd_list_length;
     return pd->cmd_list;
 }
-static SwitcherMode window_mode_result ( int mretv, G_GNUC_UNUSED char **input, unsigned int selected_line, Switcher *sw )
+static char ** window_mode_get_data_cd ( unsigned int *length, Switcher *sw )
+{
+    return _window_mode_get_data ( length, sw, TRUE );
+}
+static char ** window_mode_get_data ( unsigned int *length, Switcher *sw )
+{
+    return _window_mode_get_data ( length, sw, FALSE );
+}
+static SwitcherMode window_mode_result ( int mretv, G_GNUC_UNUSED char **input,
+                                         unsigned int selected_line,
+                                         Switcher *sw )
 {
     SwitcherModePrivateData *rmpd = (SwitcherModePrivateData *) sw->private_data;
     SwitcherMode            retv  = MODE_EXIT;
@@ -476,7 +514,7 @@ static SwitcherMode window_mode_result ( int mretv, G_GNUC_UNUSED char **input,
     else if ( ( mretv & MENU_QUICK_SWITCH ) == MENU_QUICK_SWITCH ) {
         retv = ( mretv & MENU_LOWER_MASK );
     }
-    else if ( ( mretv & ( MENU_OK | MENU_CUSTOM_INPUT ) ) && rmpd->cmd_list[selected_line] ) {
+    else if ( ( mretv & ( MENU_OK ) ) && rmpd->cmd_list[selected_line] ) {
         if ( rmpd->config_i3_mode ) {
             // Hack for i3.
             i3_support_focus_window ( rmpd->ids->array[selected_line] );
@@ -486,20 +524,17 @@ static SwitcherMode window_mode_result ( int mretv, G_GNUC_UNUSED char **input,
             Window root    = RootWindow ( display, XScreenNumberOfScreen ( screen ) );
             // Change to the desktop of the selected window/client.
             // TODO: get rid of strtol
-            window_send_message ( display, root, root, netatoms[_NET_CURRENT_DESKTOP],
-                                  strtol ( rmpd->cmd_list[selected_line], NULL, 10 ) - 1,
+            window_send_message ( display, root, root, netatoms[_NET_CURRENT_DESKTOP], strtol ( rmpd->cmd_list[selected_line], NULL, 10 ),
                                   SubstructureNotifyMask | SubstructureRedirectMask, 0 );
             XSync ( display, False );
 
-            window_send_message ( display, root, rmpd->ids->array[selected_line],
-                                  netatoms[_NET_ACTIVE_WINDOW], 2, // 2 = pager
+            window_send_message ( display, root, rmpd->ids->array[selected_line], netatoms[_NET_ACTIVE_WINDOW], 2, // 2 = pager
                                   SubstructureNotifyMask | SubstructureRedirectMask, 0 );
         }
     }
     return retv;
 }
 
-
 static void window_mode_destroy ( Switcher *sw )
 {
     SwitcherModePrivateData *rmpd = (SwitcherModePrivateData *) sw->private_data;
@@ -514,7 +549,7 @@ static void window_mode_destroy ( Switcher *sw )
     }
 }
 
-static const char *mgrv ( unsigned int selected_line, void *sw, G_GNUC_UNUSED int *state )
+static const char *mgrv ( unsigned int selected_line, void *sw, int *state )
 {
     SwitcherModePrivateData *rmpd = ( (Switcher *) sw )->private_data;
     if ( window_client ( display, rmpd->ids->array[selected_line] )->demands ) {
@@ -529,7 +564,6 @@ static const char *mgrv ( unsigned int selected_line, void *sw, G_GNUC_UNUSED in
 Switcher window_mode =
 {
     .name         = "window",
-    .tb           = NULL,
     .keycfg       = NULL,
     .keystr       = NULL,
     .modmask      = AnyModifier,
@@ -542,3 +576,21 @@ Switcher window_mode =
     .private_data = NULL,
     .free         = NULL
 };
+Switcher window_mode_cd =
+{
+    .name         = "windowcd",
+    .keycfg       = NULL,
+    .keystr       = NULL,
+    .modmask      = AnyModifier,
+    .init         = window_mode_init,
+    .get_data     = window_mode_get_data_cd,
+    .result       = window_mode_result,
+    .destroy      = window_mode_destroy,
+    .token_match  = window_match,
+    .mgrv         = mgrv,
+    .private_data = NULL,
+    .free         = NULL
+};
+
+#endif // WINDOW_MODE
+
