@@ -32,6 +32,7 @@
 #include <unistd.h>
 #include <stdint.h>
 #include <glib.h>
+#include <cairo.h>
 
 #include <X11/X.h>
 #include <X11/Xatom.h>
@@ -44,27 +45,32 @@
 #include <X11/extensions/Xinerama.h>
 
 #include <rofi.h>
-#define OVERLAP( a, b, c, d )                      ( ( ( a ) == ( c ) && ( b ) == ( d ) ) || MIN ( ( a ) + ( b ), ( c ) + ( d ) ) - MAX ( ( a ), ( c ) ) > 0 )
-#define INTERSECT( x, y, w, h, x1, y1, w1, h1 )    ( OVERLAP ( ( x ), ( w ), ( x1 ), ( w1 ) ) && OVERLAP ( ( y ), ( h ), ( y1 ), ( h1 ) ) )
+#define OVERLAP( a, b, c,                          \
+                 d )       ( ( ( a ) == ( c ) &&   \
+                               ( b ) == ( d ) ) || \
+                             MIN ( ( a ) + ( b ), ( c ) + ( d ) ) - MAX ( ( a ), ( c ) ) > 0 )
+#define INTERSECT( x, y, w, h, x1, y1, w1,                   \
+                   h1 )    ( OVERLAP ( ( x ), ( w ), ( x1 ), \
+                                       ( w1 ) ) && OVERLAP ( ( y ), ( h ), ( y1 ), ( h1 ) ) )
 #include "x11-helper.h"
 
-Atom         netatoms[NUM_NETATOMS];
-const char   *netatom_names[] = { EWMH_ATOMS ( ATOM_CHAR ) };
+Atom            netatoms[NUM_NETATOMS];
+const char      *netatom_names[] = { EWMH_ATOMS ( ATOM_CHAR ) };
 // Mask indicating num-lock.
-unsigned int NumlockMask = 0;
+unsigned int    NumlockMask = 0;
+
+extern Colormap map;
 
 // retrieve a property of any type from a window
-int window_get_prop ( Display *display, Window w, Atom prop,
-                      Atom *type, int *items,
-                      void *buffer, unsigned int bytes )
+int window_get_prop ( Display *display, Window w, Atom prop, Atom *type, int *items, void *buffer, unsigned int bytes )
 {
     int           format;
     unsigned long nitems, nbytes;
     unsigned char *ret = NULL;
     memset ( buffer, 0, bytes );
 
-    if ( XGetWindowProperty ( display, w, prop, 0, bytes / 4, False, AnyPropertyType, type,
-                              &format, &nitems, &nbytes, &ret ) == Success && ret && *type != None && format ) {
+    if ( XGetWindowProperty ( display, w, prop, 0, bytes / 4, False, AnyPropertyType, type, &format, &nitems, &nbytes, &ret ) == Success &&
+         ret && *type != None && format ) {
         if ( format == 8 ) {
             memmove ( buffer, ret, MIN ( bytes, nitems ) );
         }
@@ -137,8 +143,49 @@ int window_get_cardinal_prop ( Display *display, Window w, Atom atom, unsigned l
     Atom type; int items;
     return window_get_prop ( display, w, atom, &type, &items, list, count * sizeof ( unsigned long ) ) && type == XA_CARDINAL ? items : 0;
 }
+int monitor_get_smallest_size ( Display *display )
+{
+    int size = MIN ( WidthOfScreen ( DefaultScreenOfDisplay ( display ) ),
+                     HeightOfScreen ( DefaultScreenOfDisplay ( display ) ) );
+    // locate the current monitor
+    if ( XineramaIsActive ( display ) ) {
+        int                monitors;
+        XineramaScreenInfo *info = XineramaQueryScreens ( display, &monitors );
 
+        if ( info ) {
+            for ( int i = 0; i < monitors; i++ ) {
+                size = MIN ( info[i].width, size );
+                size = MIN ( info[i].height, size );
+            }
+        }
+        XFree ( info );
+    }
 
+    return size;
+}
+int monitor_get_dimension ( Display *display, Screen *screen, int monitor, workarea *mon )
+{
+    memset ( mon, 0, sizeof ( workarea ) );
+    mon->w = WidthOfScreen ( screen );
+    mon->h = HeightOfScreen ( screen );
+    // locate the current monitor
+    if ( XineramaIsActive ( display ) ) {
+        int                monitors;
+        XineramaScreenInfo *info = XineramaQueryScreens ( display, &monitors );
+
+        if ( info ) {
+            if ( monitor >= 0 && monitor < monitors ) {
+                mon->x = info[monitor].x_org;
+                mon->y = info[monitor].y_org;
+                mon->w = info[monitor].width;
+                mon->h = info[monitor].height;
+                return TRUE;
+            }
+            XFree ( info );
+        }
+    }
+    return FALSE;
+}
 // find the dimensions of the monitor displaying point x,y
 void monitor_dimensions ( Display *display, Screen *screen, int x, int y, workarea *mon )
 {
@@ -192,7 +239,6 @@ static int pointer_get ( Display *display, Window root, int *x, int *y )
     return 0;
 }
 
-
 // determine which monitor holds the active window, or failing that the mouse pointer
 void monitor_active ( Display *display, workarea *mon )
 {
@@ -203,15 +249,32 @@ void monitor_active ( Display *display, workarea *mon )
     Window id;
     Atom   type;
     int    count;
+    if ( config.monitor >= 0 ) {
+        if ( monitor_get_dimension ( display, screen, config.monitor, mon ) ) {
+            return;
+        }
+        fprintf ( stderr, "Failed to find selected monitor.\n" );
+    }
     if ( window_get_prop ( display, root, netatoms[_NET_ACTIVE_WINDOW], &type, &count, &id, sizeof ( Window ) )
          && type == XA_WINDOW && count > 0 ) {
         XWindowAttributes attr;
         if ( XGetWindowAttributes ( display, id, &attr ) ) {
             Window junkwin;
-            if ( XTranslateCoordinates ( display, id, attr.root,
-                                         -attr.border_width,
-                                         -attr.border_width,
-                                         &x, &y, &junkwin ) == True ) {
+            if ( XTranslateCoordinates ( display, id, attr.root, -attr.border_width, -attr.border_width, &x, &y, &junkwin ) == True ) {
+                if ( config.monitor == -2 ) {
+                    // place the menu above the window
+                    // if some window is focused, place menu above window, else fall
+                    // back to selected monitor.
+                    mon->x = x;
+                    mon->y = y;
+                    mon->w = attr.width;
+                    mon->h = attr.height;
+                    mon->t = attr.border_width;
+                    mon->b = attr.border_width;
+                    mon->l = attr.border_width;
+                    mon->r = attr.border_width;
+                    return;
+                }
                 monitor_dimensions ( display, screen, x, y, mon );
                 return;
             }
@@ -225,9 +288,7 @@ void monitor_active ( Display *display, workarea *mon )
     monitor_dimensions ( display, screen, 0, 0, mon );
 }
 
-int window_send_message ( Display *display, Window target,
-                          Window subject, Atom atom,
-                          unsigned long protocol, unsigned long mask, Time time )
+int window_send_message ( Display *display, Window trg, Window subject, Atom atom, unsigned long protocol, unsigned long mask, Time time )
 {
     XEvent e;
     memset ( &e, 0, sizeof ( XEvent ) );
@@ -238,15 +299,20 @@ int window_send_message ( Display *display, Window target,
     e.xclient.data.l[1]    = time;
     e.xclient.send_event   = True;
     e.xclient.format       = 32;
-    int r = XSendEvent ( display, target, False, mask, &e ) ? 1 : 0;
+    int r = XSendEvent ( display, trg, False, mask, &e ) ? 1 : 0;
     XFlush ( display );
     return r;
 }
 
+extern unsigned int normal_window_mode;
 int take_keyboard ( Display *display, Window w )
 {
+    if ( normal_window_mode ) {
+        return 1;
+    }
     for ( int i = 0; i < 500; i++ ) {
-        if ( XGrabKeyboard ( display, w, True, GrabModeAsync, GrabModeAsync, CurrentTime ) == GrabSuccess ) {
+        if ( XGrabKeyboard ( display, w, True, GrabModeAsync, GrabModeAsync,
+                             CurrentTime ) == GrabSuccess ) {
             return 1;
         }
         usleep ( 1000 );
@@ -257,7 +323,9 @@ int take_keyboard ( Display *display, Window w )
 
 void release_keyboard ( Display *display )
 {
-    XUngrabKeyboard ( display, CurrentTime );
+    if ( !normal_window_mode ) {
+        XUngrabKeyboard ( display, CurrentTime );
+    }
 }
 // bind a key combination on a root window, compensating for Lock* states
 void x11_grab_key ( Display *display, unsigned int modmask, KeySym key )
@@ -276,6 +344,21 @@ void x11_grab_key ( Display *display, unsigned int modmask, KeySym key )
     }
 }
 
+void x11_ungrab_key ( Display *display, unsigned int modmask, KeySym key )
+{
+    Screen  *screen = DefaultScreenOfDisplay ( display );
+    Window  root    = RootWindow ( display, XScreenNumberOfScreen ( screen ) );
+    KeyCode keycode = XKeysymToKeycode ( display, key );
+
+    // unbind to combinations of mod and lock masks, so caps and numlock don't confuse people
+    XUngrabKey ( display, keycode, modmask, root );
+    XUngrabKey ( display, keycode, modmask | LockMask, root );
+
+    if ( NumlockMask ) {
+        XUngrabKey ( display, keycode, modmask | NumlockMask, root );
+        XUngrabKey ( display, keycode, modmask | NumlockMask | LockMask, root );
+    }
+}
 /**
  * @param display The connection to the X server.
  *
@@ -345,8 +428,8 @@ void x11_parse_key ( char *combo, unsigned int *mod, KeySym *key )
     KeySym sym = XStringToKeysym ( combo + i );
 
     if ( sym == NoSymbol || ( !modmask && ( strchr ( combo, '-' ) || strchr ( combo, '+' ) ) ) ) {
+        // TODO popup
         fprintf ( stderr, "sorry, cannot understand key combination: %s\n", combo );
-        exit ( EXIT_FAILURE );
     }
 
     *key = sym;
@@ -357,8 +440,7 @@ void x11_set_window_opacity ( Display *display, Window box, unsigned int opacity
     // Scale 0-100 to 0 - UINT32_MAX.
     unsigned int opacity_set = ( unsigned int ) ( ( opacity / 100.0 ) * UINT32_MAX );
     // Set opacity.
-    XChangeProperty ( display, box, netatoms[_NET_WM_WINDOW_OPACITY],
-                      XA_CARDINAL, 32, PropModeReplace,
+    XChangeProperty ( display, box, netatoms[_NET_WM_WINDOW_OPACITY], XA_CARDINAL, 32, PropModeReplace,
                       ( unsigned char * ) &opacity_set, 1L );
 }
 
@@ -375,7 +457,6 @@ static void x11_create_frequently_used_atoms ( Display *display )
     }
 }
 
-
 static int ( *xerror )( Display *, XErrorEvent * );
 /**
  * @param d  The connection to the X server.
@@ -385,10 +466,8 @@ static int ( *xerror )( Display *, XErrorEvent * );
  */
 static int display_oops ( Display *d, XErrorEvent *ee )
 {
-    if ( ee->error_code == BadWindow
-         || ( ee->request_code == X_GrabButton && ee->error_code == BadAccess )
-         || ( ee->request_code == X_GrabKey && ee->error_code == BadAccess )
-         ) {
+    if ( ee->error_code == BadWindow || ( ee->request_code == X_GrabButton && ee->error_code == BadAccess )
+         || ( ee->request_code == X_GrabKey && ee->error_code == BadAccess ) ) {
         return 0;
     }
 
@@ -408,9 +487,6 @@ void x11_setup ( Display *display )
     x11_create_frequently_used_atoms ( display );
 }
 
-
-
-extern Colormap    map;
 extern XVisualInfo vinfo;
 int                truecolor = FALSE;
 void create_visual_and_colormap ( Display *display )
@@ -432,66 +508,143 @@ void create_visual_and_colormap ( Display *display )
         map          = DefaultColormap ( display, screen );
     }
 }
-unsigned int color_get ( Display *display, const char *const name )
+
+cairo_format_t get_format ( void )
 {
-    XColor color = { 0, 0, 0, 0, 0, 0 };
+    if ( truecolor ) {
+        return CAIRO_FORMAT_ARGB32;
+    }
+    return CAIRO_FORMAT_RGB24;
+}
+
+unsigned int color_get ( Display *display, const char *const name, const char * const defn )
+{
+    char   *copy  = g_strdup ( name );
+    char   *cname = g_strstrip ( copy );
+    XColor color  = { 0, 0, 0, 0, 0, 0 };
+    XColor def;
     // Special format.
-    if ( strncmp ( name, "argb:", 5 ) == 0 ) {
-        color.pixel = strtoul ( &name[5], NULL, 16 );
-        color.red   = ( ( color.pixel & 0x00FF0000 ) >> 16 ) * 255;
-        color.green = ( ( color.pixel & 0x0000FF00 ) >> 8  ) * 255;
-        color.blue  = ( ( color.pixel & 0x000000FF )       ) * 255;
+    if ( strncmp ( cname, "argb:", 5 ) == 0 ) {
+        color.pixel = strtoul ( &cname[5], NULL, 16 );
+        color.red   = ( ( color.pixel & 0x00FF0000 ) >> 16 ) * 256;
+        color.green = ( ( color.pixel & 0x0000FF00 ) >> 8  ) * 256;
+        color.blue  = ( ( color.pixel & 0x000000FF )       ) * 256;
         if ( !truecolor ) {
             // This will drop alpha part.
             Status st = XAllocColor ( display, map, &color );
             if ( st == None ) {
-                fprintf ( stderr, "Failed to parse color: '%s'\n", name );
+                fprintf ( stderr, "Failed to parse color: '%s'\n", cname );
+                st = XAllocNamedColor ( display, map, defn, &color, &def );
+                if ( st == None  ) {
+                    fprintf ( stderr, "Failed to allocate fallback color\n" );
+                    exit ( EXIT_FAILURE );
+                }
+            }
+        }
+    }
+    else {
+        Status st = XAllocNamedColor ( display, map, cname, &color, &def );
+        if ( st == None ) {
+            fprintf ( stderr, "Failed to parse color: '%s'\n", cname );
+            st = XAllocNamedColor ( display, map, defn, &color, &def );
+            if ( st == None  ) {
+                fprintf ( stderr, "Failed to allocate fallback color\n" );
                 exit ( EXIT_FAILURE );
             }
-            return color.pixel;
         }
-        return color.pixel;
-    }
-    else {
-        Status st = XAllocNamedColor ( display, map, name, &color, &color );
-        if ( st == None ) {
-            fprintf ( stderr, "Failed to parse color: '%s'\n", name );
-            exit ( EXIT_FAILURE );
-        }
-        return color.pixel;
     }
+    g_free ( copy );
+    return color.pixel;
 }
-
-unsigned int color_background ( Display *display )
+void x11_helper_set_cairo_rgba ( cairo_t *d, unsigned int pixel )
 {
-    if ( !config.color_enabled ) {
-        return color_get ( display, config.menu_bg );
-    }
-    else {
-        unsigned int retv = 0;
-
-        gchar        **vals = g_strsplit ( config.color_window, ",", 2 );
-        if ( vals != NULL && vals[0] != NULL ) {
-            retv = color_get ( display, g_strstrip ( vals[0] ) );
-        }
-        g_strfreev ( vals );
-        return retv;
-    }
+    cairo_set_source_rgba ( d,
+                            ( ( pixel & 0x00FF0000 ) >> 16 ) / 256.0,
+                            ( ( pixel & 0x0000FF00 ) >> 8 ) / 256.0,
+                            ( ( pixel & 0x000000FF ) >> 0 ) / 256.0,
+                            ( ( pixel & 0xFF000000 ) >> 24 ) / 256.0
+                            );
 }
-
-unsigned int color_border ( Display *display )
+/**
+ * Color cache.
+ *
+ * This stores the current color until
+ */
+enum
 {
-    if ( !config.color_enabled ) {
-        return color_get ( display, config.menu_bc );
+    BACKGROUND,
+    BORDER,
+    SEPARATOR
+};
+struct
+{
+    unsigned int color;
+    unsigned int set;
+} color_cache[3] = {
+    { 0, FALSE },
+    { 0, FALSE },
+    { 0, FALSE }
+};
+void color_cache_reset ( void )
+{
+    color_cache[BACKGROUND].set = FALSE;
+    color_cache[BORDER].set     = FALSE;
+    color_cache[SEPARATOR].set  = FALSE;
+}
+void color_background ( Display *display, cairo_t *d )
+{
+    if ( !color_cache[BACKGROUND].set ) {
+        if ( !config.color_enabled ) {
+            color_cache[BACKGROUND].color = color_get ( display, config.menu_bg, "black" );
+        }
+        else {
+            gchar **vals = g_strsplit ( config.color_window, ",", 3 );
+            if ( vals != NULL && vals[0] != NULL ) {
+                color_cache[BACKGROUND].color = color_get ( display, vals[0], "black" );
+            }
+            g_strfreev ( vals );
+        }
+        color_cache[BACKGROUND].set = TRUE;
+    }
+
+    x11_helper_set_cairo_rgba ( d, color_cache[BACKGROUND].color );
+}
+
+void color_border ( Display *display, cairo_t *d  )
+{
+    if ( !color_cache[BORDER].set ) {
+        if ( !config.color_enabled ) {
+            color_cache[BORDER].color = color_get ( display, config.menu_bc, "white" );
+        }
+        else {
+            gchar **vals = g_strsplit ( config.color_window, ",", 3 );
+            if ( vals != NULL && vals[0] != NULL && vals[1] != NULL ) {
+                color_cache[BORDER].color = color_get ( display, vals[1], "white" );
+            }
+            g_strfreev ( vals );
+        }
+        color_cache[BORDER].set = TRUE;
     }
-    else {
-        unsigned int retv = 0;
+    x11_helper_set_cairo_rgba ( d, color_cache[BORDER].color );
+}
 
-        gchar        **vals = g_strsplit ( config.color_window, ",", 2 );
-        if ( vals != NULL && vals[0] != NULL && vals[1] != NULL ) {
-            retv = color_get ( display, vals[1] );
+void color_separator ( Display *display, cairo_t *d )
+{
+    if ( !color_cache[SEPARATOR].set ) {
+        if ( !config.color_enabled ) {
+            color_cache[SEPARATOR].color = color_get ( display, config.menu_bc, "white" );
+        }
+        else {
+            gchar **vals = g_strsplit ( config.color_window, ",", 3 );
+            if ( vals != NULL && vals[0] != NULL && vals[1] != NULL && vals[2] != NULL  ) {
+                color_cache[SEPARATOR].color = color_get ( display, vals[2], "white" );
+            }
+            else if ( vals != NULL && vals[0] != NULL && vals[1] != NULL ) {
+                color_cache[SEPARATOR].color = color_get ( display, vals[1], "white" );
+            }
+            g_strfreev ( vals );
         }
-        g_strfreev ( vals );
-        return retv;
+        color_cache[SEPARATOR].set = TRUE;
     }
+    x11_helper_set_cairo_rgba ( d, color_cache[SEPARATOR].color );
 }
